# Generated by Django 5.2.1 on 2025-06-11 22:54

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("reports_dashboard", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="report",
            name="task_id",
            field=models.CharField(
                blank=True,
                editable=False,
                max_length=64,
                null=True,
                unique=True,
                verbose_name="Background Task ID",
            ),
        ),
    ]
